@@ -14,9 +14,13 @@ from dagster.core.system_config.objects import EnvironmentConfig
 from dagster.core.telemetry import log_repo_stats, telemetry_wrapper
 from dagster.core.utils import str_format_set
 from dagster.utils import merge_dicts
-from dagster.utils.backcompat import canonicalize_run_config
 
-from .context_creation_pipeline import pipeline_initialization_manager, scoped_pipeline_context
+from .context_creation_pipeline import (
+    ExecutionContextManager,
+    PipelineExecutionContextManager,
+    PlanExecutionContextManager,
+    scoped_pipeline_context,
+)
 from .results import PipelineExecutionResult
 
 ## Brief guide to the execution APIs
@@ -73,12 +77,14 @@ def execute_run_iterator(pipeline, pipeline_run, instance):
     return iter(
         _ExecuteRunWithPlanIterable(
             execution_plan=execution_plan,
-            pipeline_run=pipeline_run,
-            instance=instance,
             iterator=_pipeline_execution_iterator,
-            run_config=pipeline_run.run_config,
-            retries=None,
-            raise_on_error=False,
+            execution_context_manager=PipelineExecutionContextManager(
+                execution_plan=execution_plan,
+                pipeline_run=pipeline_run,
+                instance=instance,
+                run_config=pipeline_run.run_config,
+                raise_on_error=False,
+            ),
         )
     )
 
@@ -139,12 +145,14 @@ def execute_run(pipeline, pipeline_run, instance, raise_on_error=False):
 
     _execute_run_iterable = _ExecuteRunWithPlanIterable(
         execution_plan=execution_plan,
-        pipeline_run=pipeline_run,
-        instance=instance,
         iterator=_pipeline_execution_iterator,
-        run_config=pipeline_run.run_config,
-        retries=None,
-        raise_on_error=raise_on_error,
+        execution_context_manager=PipelineExecutionContextManager(
+            execution_plan=execution_plan,
+            pipeline_run=pipeline_run,
+            instance=instance,
+            run_config=pipeline_run.run_config,
+            raise_on_error=raise_on_error,
+        ),
     )
     event_list = list(_execute_run_iterable)
     pipeline_context = _execute_run_iterable.pipeline_context
@@ -158,6 +166,7 @@ def execute_run(pipeline, pipeline_run, instance, raise_on_error=False):
             pipeline_run.run_config,
             pipeline_run,
             instance,
+            intermediate_storage=pipeline_context.intermediates_manager,
             system_storage_data=SystemStorageData(
                 intermediates_manager=pipeline_context.intermediates_manager,
                 file_manager=pipeline_context.file_manager,
@@ -174,7 +183,6 @@ def execute_pipeline_iterator(
     tags=None,
     solid_selection=None,
     instance=None,
-    environment_dict=None,
 ):
     '''Execute a pipeline iteratively.
 
@@ -209,8 +217,6 @@ def execute_pipeline_iterator(
     Returns:
       Iterator[DagsterEvent]: The stream of events resulting from pipeline execution.
     '''
-    # stack level is to punch through helper function
-    run_config = canonicalize_run_config(run_config, environment_dict, stacklevel=4)
 
     (
         pipeline,
@@ -242,7 +248,6 @@ def execute_pipeline_iterator(
     return execute_run_iterator(pipeline, pipeline_run, instance)
 
 
-@telemetry_wrapper
 def execute_pipeline(
     pipeline,
     run_config=None,
@@ -252,7 +257,6 @@ def execute_pipeline(
     solid_selection=None,
     instance=None,
     raise_on_error=True,
-    environment_dict=None,
 ):
     '''Execute a pipeline synchronously.
 
@@ -290,8 +294,30 @@ def execute_pipeline(
     This is the entrypoint for dagster CLI execution. For the dagster-graphql entrypoint, see
     ``dagster.core.execution.api.execute_plan()``.
     '''
-    # stack level is to punch through helper function and telemetry wrapper
-    run_config = canonicalize_run_config(run_config, environment_dict, stacklevel=5)
+    instance = instance or DagsterInstance.ephemeral()
+    return _logged_execute_pipeline(
+        pipeline,
+        run_config=run_config,
+        mode=mode,
+        preset=preset,
+        tags=tags,
+        solid_selection=solid_selection,
+        instance=instance,
+        raise_on_error=raise_on_error,
+    )
+
+
+@telemetry_wrapper
+def _logged_execute_pipeline(
+    pipeline,
+    run_config=None,
+    mode=None,
+    preset=None,
+    tags=None,
+    solid_selection=None,
+    instance=None,
+    raise_on_error=True,
+):
 
     (
         pipeline,
@@ -482,12 +508,15 @@ def execute_plan_iterator(
     return iter(
         _ExecuteRunWithPlanIterable(
             execution_plan=execution_plan,
-            run_config=run_config,
-            pipeline_run=pipeline_run,
-            instance=instance,
-            retries=retries,
             iterator=inner_plan_execution_iterator,
-            raise_on_error=False,
+            execution_context_manager=PlanExecutionContextManager(
+                retries=retries,
+                execution_plan=execution_plan,
+                run_config=run_config,
+                pipeline_run=pipeline_run,
+                instance=instance,
+                raise_on_error=False,
+            ),
         )
     )
 
@@ -540,54 +569,60 @@ def create_execution_plan(pipeline, run_config=None, mode=None, step_keys_to_exe
     )
 
 
-def _pipeline_execution_iterator(pipeline_context, execution_plan, retries=None):
-    '''A complete execution of a pipeline. Yields pipeline start, success,
-    and failure events.
+class BoolRef:
+    def __init__(self, value):
+        self.value = value
 
-    Args:
-        pipeline_context (SystemPipelineExecutionContext):
-        execution_plan (ExecutionPlan):
-        retries (None): Must be None. This is to align the signature of
-            `_pipeline_execution_iterator` with that of
-            `dagster.core.execution.plan.execute_plan.inner_plan_execution_iterator` so the same
-            machinery in _ExecuteRunWithPlanIterable can call them without unpleasant workarounds.
-            (Default: None)
-    '''
-    check.inst_param(pipeline_context, 'pipeline_context', SystemPipelineExecutionContext)
-    check.inst_param(execution_plan, 'execution_plan', ExecutionPlan)
-    check.invariant(
-        retries is None, 'Programming error: Retries not supported in _pipeline_execution_iterator'
-    )
-    yield DagsterEvent.pipeline_start(pipeline_context)
 
-    steps_started = set([])
-    pipeline_success = True
-    generator_closed = False
+def _core_execution_iterator(pipeline_context, execution_plan, steps_started, pipeline_success_ref):
     try:
         for event in pipeline_context.executor.execute(pipeline_context, execution_plan):
             if event.is_step_start:
                 steps_started.add(event.step_key)
             if event.is_step_success:
                 if event.step_key not in steps_started:
-                    pipeline_success = False
+                    pipeline_success_ref.value = False
                 else:
                     steps_started.remove(event.step_key)
             if event.is_step_failure:
-                pipeline_success = False
+                pipeline_success_ref.value = False
+            yield event
+    except (Exception, KeyboardInterrupt):
+        pipeline_success_ref.value = False
+        raise  # finally block will run before this is re-raised
+
+
+def _pipeline_execution_iterator(pipeline_context, execution_plan):
+    '''A complete execution of a pipeline. Yields pipeline start, success,
+    and failure events.
+
+    Args:
+        pipeline_context (SystemPipelineExecutionContext):
+        execution_plan (ExecutionPlan):
+    '''
+    check.inst_param(pipeline_context, 'pipeline_context', SystemPipelineExecutionContext)
+    check.inst_param(execution_plan, 'execution_plan', ExecutionPlan)
+
+    yield DagsterEvent.pipeline_start(pipeline_context)
+
+    steps_started = set([])
+    pipeline_success_ref = BoolRef(True)
+    generator_closed = False
+    try:
+        for event in _core_execution_iterator(
+            pipeline_context, execution_plan, steps_started, pipeline_success_ref
+        ):
             yield event
     except GeneratorExit:
         # Shouldn't happen, but avoid runtime-exception in case this generator gets GC-ed
         # (see https://amir.rachum.com/blog/2017/03/03/generator-cleanup/).
         generator_closed = True
-        pipeline_success = False
+        pipeline_success_ref.value = False
         raise
-    except (Exception, KeyboardInterrupt):
-        pipeline_success = False
-        raise  # finally block will run before this is re-raised
     finally:
         if steps_started:
-            pipeline_success = False
-        if pipeline_success:
+            pipeline_success_ref.value = False
+        if pipeline_success_ref.value:
             event = DagsterEvent.pipeline_success(pipeline_context)
         else:
             event = DagsterEvent.pipeline_failure(pipeline_context)
@@ -604,38 +639,25 @@ class _ExecuteRunWithPlanIterable(object):
     cf. `dagster.utils.EventGenerationManager`.
     '''
 
-    def __init__(
-        self, execution_plan, pipeline_run, instance, iterator, run_config, retries, raise_on_error,
-    ):
+    def __init__(self, execution_plan, iterator, execution_context_manager):
         self.execution_plan = check.inst_param(execution_plan, 'execution_plan', ExecutionPlan)
-        self.pipeline_run = check.inst_param(pipeline_run, 'pipeline_run', PipelineRun)
-        self.instance = check.inst_param(instance, 'instance', DagsterInstance)
         self.iterator = check.callable_param(iterator, 'iterator')
-        self.run_config = (
-            check.opt_dict_param(run_config, 'run_config') or self.pipeline_run.run_config
+        self.execution_context_manager = check.inst_param(
+            execution_context_manager, 'execution_context_manager', ExecutionContextManager
         )
-        self.retries = check.opt_inst_param(retries, 'retries', Retries)
-        self.raise_on_error = check.bool_param(raise_on_error, 'raise_on_error')
+
         self.pipeline_context = None
 
     def __iter__(self):
-        initialization_manager = pipeline_initialization_manager(
-            self.execution_plan,
-            self.run_config,
-            self.pipeline_run,
-            self.instance,
-            raise_on_error=self.raise_on_error,
-        )
-        for event in initialization_manager.generate_setup_events():
+
+        for event in self.execution_context_manager.prepare_context():
             yield event
-        self.pipeline_context = initialization_manager.get_object()
+        self.pipeline_context = self.execution_context_manager.get_context()
         generator_closed = False
         try:
             if self.pipeline_context:  # False if we had a pipeline init failure
                 for event in self.iterator(
-                    execution_plan=self.execution_plan,
-                    pipeline_context=self.pipeline_context,
-                    retries=self.retries,
+                    execution_plan=self.execution_plan, pipeline_context=self.pipeline_context,
                 ):
                     yield event
         except GeneratorExit:
@@ -644,7 +666,7 @@ class _ExecuteRunWithPlanIterable(object):
             generator_closed = True
             raise
         finally:
-            for event in initialization_manager.generate_teardown_events():
+            for event in self.execution_context_manager.shutdown_context():
                 if not generator_closed:
                     yield event
 
