@@ -4,33 +4,37 @@ Not every property on these should be exposed to random Jane or Joe dagster user
 so we have a different layer of objects that encode the explicit public API
 in the user_context module
 '''
+
 from collections import namedtuple
 
 from dagster import check
 from dagster.core.definitions.executable import ExecutablePipeline
+from dagster.core.definitions.hook import HookDefinition
 from dagster.core.definitions.mode import ModeDefinition
 from dagster.core.definitions.resource import ScopedResourcesBuilder
 from dagster.core.definitions.step_launcher import StepLauncher
 from dagster.core.errors import DagsterInvariantViolationError
+from dagster.core.execution.retries import Retries
+from dagster.core.executor.base import Executor
 from dagster.core.log_manager import DagsterLogManager
 from dagster.core.storage.file_manager import FileManager
 from dagster.core.storage.pipeline_run import PipelineRun
 from dagster.core.system_config.objects import EnvironmentConfig
 
 
-class SystemPipelineExecutionContextData(
+class SystemExecutionContextData(
     namedtuple(
-        '_SystemPipelineExecutionContextData',
+        '_SystemExecutionContextData',
         (
             'pipeline_run scoped_resources_builder environment_config pipeline '
-            'mode_def system_storage_def instance intermediates_manager file_manager '
-            'executor raise_on_error'
+            'mode_def system_storage_def intermediate_storage_def instance intermediates_manager file_manager '
+            'raise_on_error retries'
         ),
     )
 ):
     '''
-    SystemPipelineExecutionContextData is the data that remains constant throughout the entire
-    execution of a pipeline.
+    SystemExecutionContextData is the data that remains constant throughout the entire
+    execution of a pipeline or plan.
     '''
 
     def __new__(
@@ -41,18 +45,19 @@ class SystemPipelineExecutionContextData(
         pipeline,
         mode_def,
         system_storage_def,
+        intermediate_storage_def,
         instance,
         intermediates_manager,
         file_manager,
-        executor,
         raise_on_error,
+        retries,
     ):
         from dagster.core.definitions.system_storage import SystemStorageDefinition
-        from dagster.core.storage.intermediates_manager import IntermediatesManager
+        from dagster.core.definitions.intermediate_storage import IntermediateStorageDefinition
+        from dagster.core.storage.intermediates_manager import IntermediateStorage
         from dagster.core.instance import DagsterInstance
-        from dagster.core.executor.base import Executor
 
-        return super(SystemPipelineExecutionContextData, cls).__new__(
+        return super(SystemExecutionContextData, cls).__new__(
             cls,
             pipeline_run=check.inst_param(pipeline_run, 'pipeline_run', PipelineRun),
             scoped_resources_builder=check.inst_param(
@@ -66,13 +71,16 @@ class SystemPipelineExecutionContextData(
             system_storage_def=check.inst_param(
                 system_storage_def, 'system_storage_def', SystemStorageDefinition
             ),
+            intermediate_storage_def=check.opt_inst_param(
+                intermediate_storage_def, 'intermediate_storage_def', IntermediateStorageDefinition
+            ),
             instance=check.inst_param(instance, 'instance', DagsterInstance),
             intermediates_manager=check.inst_param(
-                intermediates_manager, 'intermediates_manager', IntermediatesManager
+                intermediates_manager, 'intermediates_manager', IntermediateStorage
             ),
             file_manager=check.inst_param(file_manager, 'file_manager', FileManager),
-            executor=check.inst_param(executor, 'executor', Executor),
             raise_on_error=check.bool_param(raise_on_error, 'raise_on_error'),
+            retries=check.inst_param(retries, 'retries', Retries),
         )
 
     @property
@@ -88,50 +96,78 @@ class SystemPipelineExecutionContextData(
         return self.pipeline.get_definition()
 
 
-class SystemPipelineExecutionContext(object):
-    __slots__ = ['_pipeline_context_data', '_log_manager']
+class SystemExecutionContext(object):
+    __slots__ = ['_execution_context_data', '_log_manager']
 
-    def __init__(self, pipeline_context_data, log_manager):
-        self._pipeline_context_data = check.inst_param(
-            pipeline_context_data, 'pipeline_context_data', SystemPipelineExecutionContextData
+    def __init__(self, execution_context_data, log_manager):
+        self._execution_context_data = check.inst_param(
+            execution_context_data, 'execution_context_data', SystemExecutionContextData
         )
         self._log_manager = check.inst_param(log_manager, 'log_manager', DagsterLogManager)
 
-    def for_step(self, step):
-        from dagster.core.execution.plan.objects import ExecutionStep
-
-        check.inst_param(step, 'step', ExecutionStep)
-
-        return SystemStepExecutionContext(
-            self._pipeline_context_data, self._log_manager.with_tags(**step.logging_tags), step,
-        )
-
-    def for_type(self, dagster_type):
-        return TypeCheckContext(self._pipeline_context_data, self.log, dagster_type)
-
-    @property
-    def executor(self):
-        return self._pipeline_context_data.executor
-
     @property
     def pipeline_run(self):
-        return self._pipeline_context_data.pipeline_run
+        return self._execution_context_data.pipeline_run
 
     @property
     def scoped_resources_builder(self):
-        return self._pipeline_context_data.scoped_resources_builder
+        return self._execution_context_data.scoped_resources_builder
 
     @property
     def run_id(self):
-        return self._pipeline_context_data.run_id
+        return self._execution_context_data.run_id
 
     @property
     def run_config(self):
-        return self._pipeline_context_data.run_config
+        return self._execution_context_data.run_config
 
     @property
     def environment_config(self):
-        return self._pipeline_context_data.environment_config
+        return self._execution_context_data.environment_config
+
+    @property
+    def pipeline(self):
+        return self._execution_context_data.pipeline
+
+    @property
+    def pipeline_def(self):
+        return self._execution_context_data.pipeline_def
+
+    @property
+    def mode_def(self):
+        return self._execution_context_data.mode_def
+
+    @property
+    def system_storage_def(self):
+        return self._execution_context_data.system_storage_def
+
+    @property
+    def intermediate_storage_def(self):
+        return self._execution_context_data.intermediate_storage_def
+
+    @property
+    def instance(self):
+        return self._execution_context_data.instance
+
+    @property
+    def intermediates_manager(self):
+        return self._execution_context_data.intermediates_manager
+
+    @property
+    def file_manager(self):
+        return self._execution_context_data.file_manager
+
+    @property
+    def raise_on_error(self):
+        return self._execution_context_data.raise_on_error
+
+    @property
+    def retries(self):
+        return self._execution_context_data.retries
+
+    @property
+    def log(self):
+        return self._log_manager
 
     @property
     def logging_tags(self):
@@ -145,56 +181,47 @@ class SystemPipelineExecutionContext(object):
         check.str_param(key, 'key')
         return self.logging_tags.get(key)
 
-    @property
-    def pipeline(self):
-        return self._pipeline_context_data.pipeline
+    def for_step(self, step):
+        from dagster.core.execution.plan.objects import ExecutionStep
+
+        check.inst_param(step, 'step', ExecutionStep)
+
+        return SystemStepExecutionContext(
+            self._execution_context_data, self._log_manager.with_tags(**step.logging_tags), step,
+        )
+
+    def for_type(self, dagster_type):
+        return TypeCheckContext(self._execution_context_data, self.log, dagster_type)
+
+
+class SystemPipelineExecutionContext(SystemExecutionContext):
+    __slots__ = ['_executor']
+
+    def __init__(self, execution_context_data, log_manager, executor):
+        super(SystemPipelineExecutionContext, self).__init__(execution_context_data, log_manager)
+        self._executor = check.inst_param(executor, 'executor', Executor)
 
     @property
-    def pipeline_def(self):
-        return self._pipeline_context_data.pipeline_def
-
-    @property
-    def mode_def(self):
-        return self._pipeline_context_data.mode_def
-
-    @property
-    def system_storage_def(self):
-        return self._pipeline_context_data.system_storage_def
-
-    @property
-    def log(self):
-        return self._log_manager
-
-    @property
-    def instance(self):
-        return self._pipeline_context_data.instance
-
-    @property
-    def intermediates_manager(self):
-        return self._pipeline_context_data.intermediates_manager
-
-    @property
-    def file_manager(self):
-        return self._pipeline_context_data.file_manager
-
-    @property
-    def raise_on_error(self):
-        return self._pipeline_context_data.raise_on_error
+    def executor(self):
+        return self._executor
 
 
-class SystemStepExecutionContext(SystemPipelineExecutionContext):
+class SystemStepExecutionContext(SystemExecutionContext):
     __slots__ = ['_step', '_resources', '_required_resource_keys', '_step_launcher']
 
-    def __init__(self, pipeline_context_data, log_manager, step):
+    def __init__(self, execution_context_data, log_manager, step):
         from dagster.core.execution.plan.objects import ExecutionStep
         from dagster.core.execution.resources_init import get_required_resource_keys_for_step
 
         self._step = check.inst_param(step, 'step', ExecutionStep)
-        super(SystemStepExecutionContext, self).__init__(pipeline_context_data, log_manager)
+        super(SystemStepExecutionContext, self).__init__(execution_context_data, log_manager)
         self._required_resource_keys = get_required_resource_keys_for_step(
-            step, pipeline_context_data.pipeline_def, pipeline_context_data.system_storage_def,
+            step,
+            execution_context_data.pipeline_def,
+            execution_context_data.system_storage_def,
+            execution_context_data.intermediate_storage_def,
         )
-        self._resources = self._pipeline_context_data.scoped_resources_builder.build(
+        self._resources = self._execution_context_data.scoped_resources_builder.build(
             self._required_resource_keys
         )
         step_launcher_resources = [
@@ -215,7 +242,7 @@ class SystemStepExecutionContext(SystemPipelineExecutionContext):
         self._log_manager = log_manager
 
     def for_compute(self):
-        return SystemComputeExecutionContext(self._pipeline_context_data, self.log, self.step)
+        return SystemComputeExecutionContext(self._execution_context_data, self.log, self.step)
 
     @property
     def step(self):
@@ -249,6 +276,9 @@ class SystemStepExecutionContext(SystemPipelineExecutionContext):
     def log(self):
         return self._log_manager
 
+    def for_hook(self, hook_def):
+        return HookContext(self._execution_context_data, self.log, hook_def, self.step)
+
 
 class SystemComputeExecutionContext(SystemStepExecutionContext):
     @property
@@ -257,7 +287,7 @@ class SystemComputeExecutionContext(SystemStepExecutionContext):
         return solid_config.config if solid_config else None
 
 
-class TypeCheckContext(SystemPipelineExecutionContext):
+class TypeCheckContext(SystemExecutionContext):
     '''The ``context`` object available to a type check function on a DagsterType.
 
     Attributes:
@@ -266,9 +296,9 @@ class TypeCheckContext(SystemPipelineExecutionContext):
         run_id (str): The id of this pipeline run.
     '''
 
-    def __init__(self, pipeline_context_data, log_manager, dagster_type):
-        super(TypeCheckContext, self).__init__(pipeline_context_data, log_manager)
-        self._resources = self._pipeline_context_data.scoped_resources_builder.build(
+    def __init__(self, execution_context_data, log_manager, dagster_type):
+        super(TypeCheckContext, self).__init__(execution_context_data, log_manager)
+        self._resources = self._execution_context_data.scoped_resources_builder.build(
             dagster_type.required_resource_keys
         )
         self._log_manager = log_manager
@@ -276,3 +306,46 @@ class TypeCheckContext(SystemPipelineExecutionContext):
     @property
     def resources(self):
         return self._resources
+
+
+class HookContext(SystemExecutionContext):
+    '''The ``context`` object available to a hook function on an DagsterEvent.
+
+    Attributes:
+        log (DagsterLogManager): Centralized log dispatch from user code.
+        hook_def (HookDefinition): The hook that the context object belongs to.
+        step (ExecutionStep) The compute step associated with the hook.
+    '''
+
+    def __init__(self, execution_context_data, log_manager, hook_def, step):
+        from dagster.core.execution.plan.objects import ExecutionStep
+
+        super(HookContext, self).__init__(execution_context_data, log_manager)
+        self._log_manager = log_manager
+        self._hook_def = check.inst_param(hook_def, 'hook_def', HookDefinition)
+        self._step = check.inst_param(step, 'step', ExecutionStep)
+
+        self._required_resource_keys = hook_def.required_resource_keys
+        self._resources = self._execution_context_data.scoped_resources_builder.build(
+            self._required_resource_keys
+        )
+
+    @property
+    def hook_def(self):
+        return self._hook_def
+
+    @property
+    def step(self):
+        return self._step
+
+    @property
+    def solid(self):
+        return self.pipeline_def.get_solid(self._step.solid_handle)
+
+    @property
+    def resources(self):
+        return self._resources
+
+    @property
+    def required_resource_keys(self):
+        return self._required_resource_keys
