@@ -1,5 +1,6 @@
 from dagster import check
 from dagster.core.definitions import (
+    AssetMaterialization,
     ExpectationResult,
     Failure,
     Materialization,
@@ -9,12 +10,12 @@ from dagster.core.definitions import (
 )
 from dagster.core.errors import (
     DagsterExecutionStepExecutionError,
-    DagsterInputHydrationConfigError,
     DagsterInvariantViolationError,
-    DagsterOutputMaterializationError,
     DagsterStepOutputNotFoundError,
     DagsterTypeCheckDidNotPass,
     DagsterTypeCheckError,
+    DagsterTypeLoadingError,
+    DagsterTypeMaterializationError,
     user_code_error_boundary,
 )
 from dagster.core.events import DagsterEvent
@@ -271,7 +272,7 @@ def core_dagster_event_sequence_for_step(step_context, prior_attempt_count):
             if isinstance(user_event, Output):
                 for evt in _create_step_events_for_output(step_context, user_event):
                     yield evt
-            elif isinstance(user_event, Materialization):
+            elif isinstance(user_event, (AssetMaterialization, Materialization)):
                 yield DagsterEvent.step_materialization(step_context, user_event)
             elif isinstance(user_event, ExpectationResult):
                 yield DagsterEvent.step_expectation_result(step_context, user_event)
@@ -337,7 +338,7 @@ def _create_output_materializations(step_context, output_name, value):
             if config_output_name == output_name:
                 step_output = step.step_output_named(output_name)
                 with user_code_error_boundary(
-                    DagsterOutputMaterializationError,
+                    DagsterTypeMaterializationError,
                     msg_fn=lambda: '''Error occurred during output materialization:
                     output name: "{output_name}"
                     step key: "{key}"
@@ -350,17 +351,17 @@ def _create_output_materializations(step_context, output_name, value):
                         solid=step_context.solid.name,
                     ),
                 ):
-                    materializations = step_output.dagster_type.output_materialization_config.materialize_runtime_values(
+                    materializations = step_output.dagster_type.materializer.materialize_runtime_values(
                         step_context, output_spec, value
                     )
 
                 for materialization in materializations:
-                    if not isinstance(materialization, Materialization):
+                    if not isinstance(materialization, (AssetMaterialization, Materialization)):
                         raise DagsterInvariantViolationError(
                             (
                                 'materialize_runtime_values on type {type_name} has returned '
-                                'value {value} of type {python_type}. You must return a '
-                                'Materialization.'
+                                'value {value} of type {python_type}. You must return an '
+                                'AssetMaterialization.'
                             ).format(
                                 type_name=step_output.dagster_type.name,
                                 value=repr(materialization),
@@ -400,7 +401,7 @@ def _user_event_sequence_for_step_compute_fn(step_context, evaluated_inputs):
 
 
 def _generate_error_boundary_msg_for_step_input(context, input_):
-    return lambda: '''Error occurred during input hydration:
+    return lambda: '''Error occurred during input loading:
     input name: "{input_}"
     step key: "{key}"
     solid invocation: "{solid}"
@@ -428,6 +429,7 @@ def _input_values_from_intermediates_manager(step_context):
                 dagster_type = step_input.dagster_type.inner_type
             else:  # This is the case where the fan-in is typed Any
                 dagster_type = step_input.dagster_type
+
             input_value = [
                 step_context.intermediates_manager.get_intermediate(
                     context=step_context,
@@ -435,7 +437,10 @@ def _input_values_from_intermediates_manager(step_context):
                     dagster_type=dagster_type,
                 )
                 for source_handle in step_input.source_handles
+                # Filter out missing intermediates from skipped upstream outputs
+                if step_context.intermediates_manager.has_intermediate(step_context, source_handle)
             ]
+
             # When we're using an object store-backed intermediate store, we wrap the
             # ObjectStoreOperation[] representing the fan-in values in a MultipleStepOutputsListWrapper
             # so we can yield the relevant object store events and unpack the values in the caller
@@ -451,10 +456,10 @@ def _input_values_from_intermediates_manager(step_context):
 
         elif step_input.is_from_config:
             with user_code_error_boundary(
-                DagsterInputHydrationConfigError,
+                DagsterTypeLoadingError,
                 msg_fn=_generate_error_boundary_msg_for_step_input(step_context, step_input),
             ):
-                input_value = step_input.dagster_type.input_hydration_config.construct_from_config_value(
+                input_value = step_input.dagster_type.loader.construct_from_config_value(
                     step_context, step_input.config_data
                 )
 
